@@ -32,6 +32,7 @@
 (require 'jka-compr)
 (require 'bookmark)
 (require 'password-cache)
+(require 'cl-macs)
 
 (declare-function cua-copy-region "cua-base")
 (declare-function pdf-tools-pdf-buffer-p "pdf-tools")
@@ -132,6 +133,29 @@ Nevertheless, this seems to work well in most cases."
   :group 'pdf-view
   :type 'boolean)
 
+(defcustom pdf-view-midnight-gamma 1.0
+  "In midnight mode, nonlinearly scale lightness.
+
+Values less than 1 increase lightness, values more than 1 decrease
+lightness (unless `pdf-view-midnight-gamma-before-invert' is non-nil, in
+which reverses the effect direction)."
+  :group 'pdf-view
+  :type 'float)
+
+(defcustom pdf-view-midnight-gamma-before-invert nil
+  "In midnight mode, whether to scale lightness before inverting.
+
+If non-nil, this inverts the direction of the effect of
+`pdf-view-midnight-gamma', i.e. values more than 1 increase lightness
+instead of decreasing it. This option is provided because it results in
+different behaviors near the ends of the lightness scale. For example,
+if this option is nil (the default), then a gamma values less than 1
+significantly lighten colors very close to black. One gets a less
+extreme effect by setting this option to non-nil and using gamma values
+greater than 1."
+  :group 'pdf-view
+  :type 'boolean)
+
 (defcustom pdf-view-change-page-hook nil
   "Hook run after changing to another page, but before displaying it.
 
@@ -212,6 +236,16 @@ Must be one of `glyph', `word', or `line'."
                  (const word)
                  (const line)))
 
+(defcustom pdf-view-mode-line-position-prefix "P"
+  "Prefix for page number shown in the mode line."
+  :group 'pdf-view
+  :type 'string)
+
+(defcustom pdf-view-mode-line-position-use-labels nil
+  "Whether current page should come from page labels."
+  :group 'pdf-view
+  :type 'boolean)
+
 
 ;; * ================================================================== *
 ;; * Internal variables and macros
@@ -271,7 +305,7 @@ regarding display of the region in the later function.")
 (defvar pdf-view-mode-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map image-mode-map)
-    (define-key map (kbd "Q")         'kill-this-buffer)
+    (define-key map (kbd "Q")         'kill-current-buffer)
     ;; Navigation in the document
     (define-key map (kbd "n")         'pdf-view-next-page-command)
     (define-key map (kbd "p")         'pdf-view-previous-page-command)
@@ -280,6 +314,8 @@ regarding display of the region in the later function.")
     (define-key map [remap forward-page]  'pdf-view-next-page-command)
     (define-key map [remap backward-page] 'pdf-view-previous-page-command)
     (define-key map (kbd "SPC")       'pdf-view-scroll-up-or-next-page)
+    (define-key map [remap scroll-up-command] #'pdf-view-scroll-up-or-next-page)
+    (define-key map [remap scroll-down-command] #'pdf-view-scroll-down-or-previous-page)
     (define-key map (kbd "S-SPC")     'pdf-view-scroll-down-or-previous-page)
     (define-key map (kbd "DEL")       'pdf-view-scroll-down-or-previous-page)
     (define-key map (kbd "C-n")       'pdf-view-next-line-or-next-page)
@@ -308,6 +344,7 @@ regarding display of the region in the later function.")
     (define-key map (kbd "s m")       'pdf-view-set-slice-using-mouse)
     (define-key map (kbd "s b")       'pdf-view-set-slice-from-bounding-box)
     (define-key map (kbd "s r")       'pdf-view-reset-slice)
+    (define-key map (kbd "s c")       'pdf-view-set-slice-common-bounding-box)
     ;; Rotation.
     (define-key map (kbd "R")              #'pdf-view-rotate)
     ;; Reconvert
@@ -364,6 +401,14 @@ PNG images in Emacs buffers."
       (setq-local mwheel-scroll-down-function
                   #'pdf-view-scroll-down-or-previous-page))
 
+  (if (boundp 'mwheel-scroll-left-function)
+      (setq-local mwheel-scroll-left-function
+                  #'image-scroll-left))
+
+  (if (boundp 'mwheel-scroll-right-function)
+      (setq-local mwheel-scroll-right-function
+                  #'image-scroll-right))
+
   ;; Disable pixel-scroll-precision-mode locally if enabled
   (if (bound-and-true-p pixel-scroll-precision-mode)
       (set (make-local-variable 'pixel-scroll-precision-mode) nil))
@@ -388,7 +433,13 @@ PNG images in Emacs buffers."
 
   ;; Setup other local variables.
   (setq-local mode-line-position
-              '(" P" (:eval (number-to-string (pdf-view-current-page)))
+              '(" " pdf-view-mode-line-position-prefix
+                ;; Show page label when enabled and available,
+                ;; otherwise show numeric page. Guard against errors.
+                (:eval
+                 (or (and pdf-view-mode-line-position-use-labels
+                          (ignore-errors (pdf-view-current-pagelabel)))
+                     (number-to-string (pdf-view-current-page))))
                 ;; Avoid errors during redisplay.
                 "/" (:eval (or (ignore-errors
                                  (number-to-string (pdf-cache-number-of-pages)))
@@ -928,6 +979,14 @@ dragging it to its bottom-right corner.  See also
             (cons (/ 1.0 (float (car size)))
                   (/ 1.0 (float (cdr size))))))))
 
+(defun pdf-view--bounding-box-to-slice (bb)
+  "Convert a '(LEFT TOP RIGHT BOTTOM) bounding box to '(X Y WIDTH HEIGHT), the format accepted by `pdf-view-set-slice'."
+  (let* ((margin (max 0 (or pdf-view-bounding-box-margin 0)))
+         (halfmargin (/ margin 2)))
+    (cl-destructuring-bind (left top right bottom) bb
+      (list (- left halfmargin) (- top halfmargin)
+            (+ (- right left) margin) (+ (- bottom top) margin)))))
+
 (defun pdf-view-set-slice-from-bounding-box (&optional window)
   "Set the slice from the page's bounding-box.
 
@@ -940,18 +999,38 @@ much more accurate than could be done manually using
 See also `pdf-view-bounding-box-margin'."
   (interactive)
   (let* ((bb (pdf-cache-boundingbox (pdf-view-current-page window)))
-         (margin (max 0 (or pdf-view-bounding-box-margin 0)))
-         (slice (list (- (nth 0 bb)
-                         (/ margin 2.0))
-                      (- (nth 1 bb)
-                         (/ margin 2.0))
-                      (+ (- (nth 2 bb) (nth 0 bb))
-                         margin)
-                      (+ (- (nth 3 bb) (nth 1 bb))
-                         margin))))
+         (slice (pdf-view--bounding-box-to-slice bb)))
     (apply 'pdf-view-set-slice
            (append slice (and window (list window))))))
 
+(defun pdf-document-common-bounding-box (&optional file-or-buffer)
+  "Return the common bounding for all pages in document FILE-OR-BUFFER, as '(LEFT TOP RIGHT BOTTOM)."
+  (let ((left 1.0)
+        (top 1.0)
+        (right 0.0)
+        (bottom 0.0))
+    (dotimes (i (pdf-info-number-of-pages file-or-buffer))
+      (cl-destructuring-bind (b-left b-top b-right b-bottom) (pdf-info-boundingbox (1+ i) file-or-buffer)
+        (setq left (min left b-left)
+              top (min top b-top)
+              right (max right b-right)
+              bottom (max bottom b-bottom))))
+    (list left top right bottom)))
+
+(defun pdf-view-set-slice-common-bounding-box (&optional window)
+  "Set the slice from the common bounding box, combined from all pages.
+
+The bounding box is calculated using the document in WINDOW, which defaults to `selected-window`.
+
+A margin is added from `pdf-view-bounding-box-margin'."
+  (interactive)
+  (let* ((bb (pdf-document-common-bounding-box))
+         (slice (pdf-view--bounding-box-to-slice bb)))
+    (when window
+      (push slice window))
+    (message (prin1-to-string slice))
+    (apply 'pdf-view-set-slice slice)))
+
 (defun pdf-view-reset-slice (&optional window)
   "Reset the current slice and redisplay WINDOW.
 
@@ -1283,6 +1362,8 @@ The colors are determined by the variable
                   (pdf-info-setoptions
                    :render/foreground (or (car pdf-view-midnight-colors) "black")
                    :render/background (or (cdr pdf-view-midnight-colors) "white")
+                   :render/gamma pdf-view-midnight-gamma
+                   :render/gammabeforeinvert pdf-view-midnight-gamma-before-invert
                    :render/usecolors
                    (if pdf-view-midnight-invert
                        ;; If midnight invert is enabled, pass "2" indicating
